import {
  takeEvery,
  put,
  call,
} from 'redux-saga/effects';
import fetch from 'node-fetch';

import * as ui from '../ui';

import check from '../check';
import root from '../root';

let myRedux;

export const states = {
  IDLE: 0,
  LOADING: 1,
};

function* doDewritoJsonUpdateRequest() {
  console.debug('mastersync doDewritoJsonUpdateRequest saga called');
  try {
    const response = yield call(fetch, 'https://raw.githubusercontent.com/ElDewrito/ElDorito/master/dist/mods/dewrito.json');
    if (response.status !== 200) {
      throw new Error(`unexpected HTTP error: ${response.status} ${response.statusText}`);
    }
    const dewritoJson = yield call(response.json.bind(response));
    yield put(myRedux.actions.updateDewritoJsonFinished({
      dewritoJson,
    }));
  } catch (err) {
    yield put(myRedux.actions.updateDewritoJsonFinished({
      err,
    }));
  }
  console.debug('mastersync doDewritoJsonUpdateRequest: done');
}

function* storeMasters({
  payload: {
    err,
    dewritoJson,
  },
}) {
  console.debug('mastersync storeMasters saga called');
  if (err) {
    yield put(ui.actions.add({
      text: `Failed to fetch master server information: ${err.message}`,
      type: ui.messageTypes.ERROR,
    }));
    return;
  }
  const {
    masterServers,
  } = dewritoJson;
  yield put(myRedux.actions.setMasters({ masters: masterServers }));
  console.debug('mastersync storeMasters: done');
}

export const initialState = {
  state: states.IDLE,
  masters: [],
  caches: [],
};

myRedux = root.sub('mastersync').all({
  initialState,

  actions: {
    updateDewritoJson: {},

    updateDewritoJsonStarted: {
      reducer: state => ({
        ...state,
        state: states.LOADING,
      }),
    },

    updateDewritoJsonFinished: {
      params: {
        err: v => v == null || v instanceof Error,
        dewritoJson: v => v === undefined ||
          check.like(v, {
            masterServers: [
              {
                list: '',
                announce: '',
                stats: '',
              },
            ],
          }),
      },
      reducer: state => ({
        ...state,
        state: states.IDLE,
      }),
    },

    setMasters: {
      params: {
        masters: v => check.array(v),
      },
      reducer: (state, { masters }) => ({
        ...state,
        masters,
      }),
    },

    setCaches: {
      params: {
        masters: v => check.array(v),
      },
      reducer: (state, { caches }) => ({
        ...state,
        caches,
      }),
    },
  },
  // selectors are all function(state, ...parameters)
  selectors: {
    hasMasters: ({ masters }) => masters.length > 0,
    getMasters: ({ masters }) => ([...masters]),
    hasCaches: ({ caches }) => caches.length > 0,
    getCaches: ({ caches }) => ([...caches]),
  },
});

export const sagas = [
  takeEvery(myRedux.actionTypes.updateDewritoJson, doDewritoJsonUpdateRequest),
  takeEvery(myRedux.actionTypes.updateDewritoJsonFinished, storeMasters),
];

export const {
  actions,
  actionTypes,
  reducer,
  selectors,
} = myRedux;

